@@ -1,3 +1,6 @@
+#ifndef CONFIGMS_H_INCLUDED
+#define CONFIGMS_H_INCLUDED
+
 /* The number of bytes in a double.  */
 #define SIZEOF_DOUBLE 8
 
@@ -40,18 +43,9 @@
 /* Name of package */
 #define PACKAGE "lame"
 
-/* Version number of package */
-#define VERSION "3.97"
-
 /* Define if compiler has function prototypes */
 #define PROTOTYPES 1
 
-/* enable VBR bitrate histogram */
-#define BRHIST 1
-
-/* IEEE754 compatible machine */
-#define TAKEHIRO_IEEE754_HACK 1
-
 /* faster log implementation with less but enough precission */
 #define USE_FAST_LOG 1
 
@@ -118,3 +112,9 @@ void acm_Free( void * mem);
         #endif
     #endif
 #endif
+
+#if defined(_M_X64) && !defined(HAVE_XMMINTRIN_H)
+        #define HAVE_XMMINTRIN_H
+#endif
+
+#endif
