@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: VbrTag.c,v 1.103 2011/05/07 16:05:17 rbrito Exp $ */
+/* $Id: VbrTag.c,v 1.106 2017/08/06 18:15:47 robert Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -65,7 +65,7 @@
 #define XING_BITRATE2  64
 #define XING_BITRATE25 32
 
-
+extern const char* get_lame_tag_encoder_short_version(void);
 
 static const char VBRTag0[] = { "Xing" };
 static const char VBRTag1[] = { "Info" };
@@ -548,7 +548,7 @@ InitVbrTag(lame_global_flags * gfp)
     gfc->VBR_seek_table.pos = 0;
 
     if (gfc->VBR_seek_table.bag == NULL) {
-        gfc->VBR_seek_table.bag = malloc(400 * sizeof(int));
+        gfc->VBR_seek_table.bag = lame_calloc(int, 400);
         if (gfc->VBR_seek_table.bag != NULL) {
             gfc->VBR_seek_table.size = 400;
         }
@@ -629,7 +629,17 @@ PutLameVBR(lame_global_flags const *gfp, size_t nMusicLength, uint8_t * pbtStrea
     int     nQuality = (100 - 10 * gfp->VBR_q - gfp->quality);
 
 
-    const char *szVersion = get_lame_very_short_version();
+    /*
+    NOTE:
+            Even though the specification for the LAME VBR tag
+            did explicitly mention other encoders than LAME,
+            many SW/HW decoder seem to be able to make use of
+            this tag only, if the encoder version starts with LAME.
+            To be compatible with such decoders, ANY encoder will
+            be forced to write a fake LAME version string!
+            As a result, the encoder version info becomes worthless.
+    */
+    const char *szVersion = get_lame_tag_encoder_short_version();
     uint8_t nVBR;
     uint8_t nRevision = 0x00;
     uint8_t nRevMethod;
@@ -902,7 +912,7 @@ lame_get_lametag_frame(lame_global_flags const *gfp, unsigned char *buffer, size
     if (gfc == 0) {
         return 0;
     }
-    if (gfc->class_id != LAME_ID) {
+    if (!is_lame_internal_flags_valid(gfc)) {
         return 0;
     }
     cfg = &gfc->cfg;
