@@ -2,7 +2,7 @@
  * id3tag.c -- Write ID3 version 1 and 2 tags.
  *
  * Copyright (C) 2000 Don Melton
- * Copyright (C) 2011 Robert Hegemann
+ * Copyright (C) 2011-2017 Robert Hegemann
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -29,7 +29,7 @@
  * NOTE: See http://id3.org/ for more information about ID3 tag formats.
  */
 
-/* $Id: id3tag.c,v 1.75 2011/11/01 19:50:23 robert Exp $ */
+/* $Id: id3tag.c,v 1.80 2017/08/28 15:39:51 robert Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -147,7 +147,10 @@ typedef enum MiscIDs { ID_TXXX = FRAME_ID('T', 'X', 'X', 'X')
         , ID_ENCR = FRAME_ID('E', 'N', 'C', 'R')
         , ID_GRID = FRAME_ID('G', 'R', 'I', 'D')
         , ID_PRIV = FRAME_ID('P', 'R', 'I', 'V')
-        , ID_VSLT = FRAME_ID('V', 'S', 'L', 'T') /* full text string */
+        , ID_USLT = FRAME_ID('U', 'S', 'L', 'T') /* full text string */
+        , ID_USER = FRAME_ID('U', 'S', 'E', 'R') /* full text string */
+        , ID_PCST = FRAME_ID('P', 'C', 'S', 'T') /* iTunes Podcast indicator, only presence important */
+        , ID_WFED = FRAME_ID('W', 'F', 'E', 'D') /* iTunes Podcast URL as TEXT FRAME !!! violates standard */
 } MiscIDs;
 
 
@@ -191,29 +194,35 @@ debug_tag_spec_flags(lame_internal_flags * gfc, const char* info)
 }
 #endif
 
-
+static int
+is_lame_internal_flags_null(lame_t gfp)
+{
+    return (gfp && gfp->internal_flags) ? 0 : 1;
+}
 
 static int
-id3v2_add_ucs2(lame_internal_flags * gfc, uint32_t frame_id, char const *lang,
-               unsigned short const *desc, unsigned short const *text);
+id3v2_add_ucs2_lng(lame_t gfp, uint32_t frame_id, unsigned short const *desc, unsigned short const *text);
 static int
-id3v2_add_latin1(lame_internal_flags * gfc, uint32_t frame_id, char const *lang, char const *desc,
-                 char const *text);
+id3v2_add_latin1_lng(lame_t gfp, uint32_t frame_id, char const *desc, char const *text);
+
 
 static void
-copyV1ToV2(lame_internal_flags * gfc, int frame_id, char const *s)
+copyV1ToV2(lame_t gfp, int frame_id, char const *s)
 {
-    unsigned int flags = gfc->tag_spec.flags;
-    id3v2_add_latin1(gfc, frame_id, 0, 0, s);
-    gfc->tag_spec.flags = flags;
+    lame_internal_flags *gfc = gfp != 0 ? gfp->internal_flags : 0;
+    if (gfc != 0) {
+        unsigned int flags = gfc->tag_spec.flags;
+        id3v2_add_latin1_lng(gfp, frame_id, 0, s);
+        gfc->tag_spec.flags = flags;
 #if 0
-    debug_tag_spec_flags(gfc, "copyV1ToV2");
+        debug_tag_spec_flags(gfc, "copyV1ToV2");
 #endif
+    }
 }
 
 
 static void
-id3v2AddLameVersion(lame_internal_flags * gfc)
+id3v2AddLameVersion(lame_t gfp)
 {
     char    buffer[1024];
     const char *b = get_lame_os_bitness();
@@ -227,13 +236,13 @@ id3v2AddLameVersion(lame_internal_flags * gfc)
     else {
         sprintf(buffer, "LAME version %s (%s)", v, u);
     }
-    copyV1ToV2(gfc, ID_ENCODER, buffer);
+    copyV1ToV2(gfp, ID_ENCODER, buffer);
 }
 
 static void
-id3v2AddAudioDuration(lame_internal_flags * gfc, double ms)
+id3v2AddAudioDuration(lame_t gfp, double ms)
 {
-    SessionConfig_t const *const cfg = &gfc->cfg;
+    SessionConfig_t const *const cfg = &gfp->internal_flags->cfg; /* caller checked pointers */
     char    buffer[1024];
     double const max_ulong = MAX_U_32_NUM;
     unsigned long playlength_ms;
@@ -250,7 +259,7 @@ id3v2AddAudioDuration(lame_internal_flags * gfc, double ms)
         playlength_ms = ms;
     }
     sprintf(buffer, "%lu", playlength_ms);
-    copyV1ToV2(gfc, ID_PLAYLENGTH, buffer);
+    copyV1ToV2(gfp, ID_PLAYLENGTH, buffer);
 }
 
 void
@@ -272,64 +281,94 @@ id3tag_genre_list(void (*handler) (int, const char *, void *), void *cookie)
 
 
 void
-id3tag_init(lame_global_flags * gfp)
+id3tag_init(lame_t gfp)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
+    lame_internal_flags *gfc = 0;
+
+    if (is_lame_internal_flags_null(gfp)) {
+        return;
+    }
+    gfc = gfp->internal_flags;
     free_id3tag(gfc);
     memset(&gfc->tag_spec, 0, sizeof gfc->tag_spec);
     gfc->tag_spec.genre_id3v1 = GENRE_NUM_UNKNOWN;
     gfc->tag_spec.padding_size = 128;
-    id3v2AddLameVersion(gfc);
+    id3v2AddLameVersion(gfp);
 }
 
 
 
 void
-id3tag_add_v2(lame_global_flags * gfp)
+id3tag_add_v2(lame_t gfp)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
+    lame_internal_flags *gfc = 0;
+
+    if (is_lame_internal_flags_null(gfp)) {
+        return;
+    }
+    gfc = gfp->internal_flags;
     gfc->tag_spec.flags &= ~V1_ONLY_FLAG;
     gfc->tag_spec.flags |= ADD_V2_FLAG;
 }
 
 void
-id3tag_v1_only(lame_global_flags * gfp)
+id3tag_v1_only(lame_t gfp)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
+    lame_internal_flags *gfc = 0;
+
+    if (is_lame_internal_flags_null(gfp)) {
+        return;
+    }
+    gfc = gfp->internal_flags;
     gfc->tag_spec.flags &= ~(ADD_V2_FLAG | V2_ONLY_FLAG);
     gfc->tag_spec.flags |= V1_ONLY_FLAG;
 }
 
 void
-id3tag_v2_only(lame_global_flags * gfp)
+id3tag_v2_only(lame_t gfp)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
+    lame_internal_flags *gfc = 0;
+
+    if (is_lame_internal_flags_null(gfp)) {
+        return;
+    }
+    gfc = gfp->internal_flags;
     gfc->tag_spec.flags &= ~V1_ONLY_FLAG;
     gfc->tag_spec.flags |= V2_ONLY_FLAG;
 }
 
 void
-id3tag_space_v1(lame_global_flags * gfp)
+id3tag_space_v1(lame_t gfp)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
+    lame_internal_flags *gfc = 0;
+
+    if (is_lame_internal_flags_null(gfp)) {
+        return;
+    }
+    gfc = gfp->internal_flags;
     gfc->tag_spec.flags &= ~V2_ONLY_FLAG;
     gfc->tag_spec.flags |= SPACE_V1_FLAG;
 }
 
 void
-id3tag_pad_v2(lame_global_flags * gfp)
+id3tag_pad_v2(lame_t gfp)
 {
     id3tag_set_pad(gfp, 128);
 }
 
 void
-id3tag_set_pad(lame_global_flags * gfp, size_t n)
+id3tag_set_pad(lame_t gfp, size_t n)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
+    lame_internal_flags *gfc = 0;
+
+    if (is_lame_internal_flags_null(gfp)) {
+        return;
+    }
+    gfc = gfp->internal_flags;
     gfc->tag_spec.flags &= ~V1_ONLY_FLAG;
     gfc->tag_spec.flags |= PAD_V2_FLAG;
     gfc->tag_spec.flags |= ADD_V2_FLAG;
-    gfc->tag_spec.padding_size = n;
+    gfc->tag_spec.padding_size = (unsigned int)n;
 }
 
 static int
@@ -382,7 +421,7 @@ local_strdup(char **dst, const char *src)
         }
         if (n > 0) {    /* string length without zero termination */
             assert(sizeof(*src) == sizeof(**dst));
-            *dst = malloc((n + 1) * sizeof(**dst));
+            *dst = lame_calloc(char, n + 1);
             if (*dst != 0) {
                 memcpy(*dst, src, n * sizeof(**dst));
                 (*dst)[n] = 0;
@@ -408,7 +447,7 @@ local_ucs2_strdup(unsigned short **dst, unsigned short const *src)
         if (n > 0) {    /* string length without zero termination */
             assert(sizeof(*src) >= 2);
             assert(sizeof(*src) == sizeof(**dst));
-            *dst = malloc((n + 1) * sizeof(**dst));
+            *dst = lame_calloc(unsigned short, n + 1);
             if (*dst != 0) {
                 memcpy(*dst, src, n * sizeof(**dst));
                 (*dst)[n] = 0;
@@ -438,7 +477,7 @@ local_ucs2_substr(unsigned short** dst, unsigned short const* src, size_t start,
 {
     size_t const len = 1 + 1 + ((start < end) ? (end - start) : 0);
     size_t n = 0;
-    unsigned short *ptr = calloc(len, sizeof(ptr[0]));
+    unsigned short *ptr = lame_calloc(unsigned short, len);
     *dst = ptr;
     if (ptr == 0 || src == 0) {
         return 0;
@@ -468,6 +507,100 @@ local_ucs2_pos(unsigned short const* str, unsigned short c)
     return -1;
 }
 
+static int
+local_char_pos(char const* str, char c)
+{
+    int     i;
+    for (i = 0; str != 0 && str[i] != 0; ++i) {
+        if (str[i] == c) {
+            return i;
+        }
+    }
+    return -1;
+}
+
+static int
+maybeLatin1(unsigned short const* text)
+{
+    if (text) {
+        unsigned short bom = *text++;
+        while (*text) {
+            unsigned short c = toLittleEndian(bom, *text++);
+            if (c > 0x00fe) return 0;
+        }
+    }
+    return 1;
+}
+
+static int searchGenre(char const* genre);
+static int sloppySearchGenre(char const* genre);
+
+static int
+lookupGenre(char const* genre)
+{
+    char   *str;
+    int     num = strtol(genre, &str, 10);
+    /* is the input a string or a valid number? */
+    if (*str) {
+        num = searchGenre(genre);
+        if (num == GENRE_NAME_COUNT) {
+            num = sloppySearchGenre(genre);
+        }
+        if (num == GENRE_NAME_COUNT) {
+            return -2; /* no common genre text found */
+        }
+    }
+    else {
+        if ((num < 0) || (num >= GENRE_NAME_COUNT)) {
+            return -1; /* number unknown */
+        }
+    }
+    return num;
+}
+
+static unsigned char *
+writeLoBytes(unsigned char *frame, unsigned short const *str, size_t n);
+
+static char*
+local_strdup_utf16_to_latin1(unsigned short const* utf16)
+{
+    size_t  len = local_ucs2_strlen(utf16);
+    unsigned char* latin1 = lame_calloc(unsigned char, len+1);
+    writeLoBytes(latin1, utf16, len);
+    return (char*)latin1;
+}
+
+
+static int
+id3tag_set_genre_utf16(lame_t gfp, unsigned short const* text)
+{
+    lame_internal_flags* gfc = gfp->internal_flags;
+    int   ret;
+    if (text == 0) {
+        return -3;
+    }
+    if (!hasUcs2ByteOrderMarker(text[0])) {
+        return -3;
+    }
+    if (maybeLatin1(text)) {
+        char*   latin1 = local_strdup_utf16_to_latin1(text);
+        int     num = lookupGenre(latin1);
+        free(latin1);
+        if (num == -1) return -1; /* number out of range */
+        if (num >= 0) {           /* common genre found  */
+            gfc->tag_spec.flags |= CHANGED_FLAG;
+            gfc->tag_spec.genre_id3v1 = num;
+            copyV1ToV2(gfp, ID_GENRE, genre_names[num]);
+            return 0;
+        }
+    }
+    ret = id3v2_add_ucs2_lng(gfp, ID_GENRE, 0, text);
+    if (ret == 0) {
+        gfc->tag_spec.flags |= CHANGED_FLAG;
+        gfc->tag_spec.genre_id3v1 = GENRE_INDEX_OTHER;
+    }
+    return ret;
+}
 
 /*
 Some existing options for ID3 tag can be specified by --tv option
@@ -481,24 +614,31 @@ as follows.
 (although some are not exactly same)*/
 
 int
-id3tag_set_albumart(lame_global_flags * gfp, const char *image, size_t size)
+id3tag_set_albumart(lame_t gfp, const char *image, size_t size)
 {
-    int     mimetype = 0;
-    unsigned char const *data = (unsigned char const *) image;
-    lame_internal_flags *gfc = gfp->internal_flags;
+    int     mimetype = MIMETYPE_NONE;
+    lame_internal_flags *gfc = 0;
 
-    /* determine MIME type from the actual image data */
-    if (2 < size && data[0] == 0xFF && data[1] == 0xD8) {
-        mimetype = MIMETYPE_JPEG;
+    if (is_lame_internal_flags_null(gfp)) {
+        return 0;
     }
-    else if (4 < size && data[0] == 0x89 && strncmp((const char *) &data[1], "PNG", 3) == 0) {
-        mimetype = MIMETYPE_PNG;
-    }
-    else if (4 < size && strncmp((const char *) data, "GIF8", 4) == 0) {
-        mimetype = MIMETYPE_GIF;
-    }
-    else {
-        return -1;
+    gfc = gfp->internal_flags;
+
+    if (image != 0) {
+        unsigned char const *data = (unsigned char const *) image;
+        /* determine MIME type from the actual image data */
+        if (2 < size && data[0] == 0xFF && data[1] == 0xD8) {
+            mimetype = MIMETYPE_JPEG;
+        }
+        else if (4 < size && data[0] == 0x89 && strncmp((const char *) &data[1], "PNG", 3) == 0) {
+            mimetype = MIMETYPE_PNG;
+        }
+        else if (4 < size && strncmp((const char *) data, "GIF8", 4) == 0) {
+            mimetype = MIMETYPE_GIF;
+        }
+        else {
+            return -1;
+        }
     }
     if (gfc->tag_spec.albumart != 0) {
         free(gfc->tag_spec.albumart);
@@ -506,13 +646,13 @@ id3tag_set_albumart(lame_global_flags * gfp, const char *image, size_t size)
         gfc->tag_spec.albumart_size = 0;
         gfc->tag_spec.albumart_mimetype = MIMETYPE_NONE;
     }
-    if (size < 1) {
+    if (size < 1 || mimetype == MIMETYPE_NONE) {
         return 0;
     }
-    gfc->tag_spec.albumart = malloc(size);
+    gfc->tag_spec.albumart = lame_calloc(unsigned char, size);
     if (gfc->tag_spec.albumart != 0) {
         memcpy(gfc->tag_spec.albumart, image, size);
-        gfc->tag_spec.albumart_size = size;
+        gfc->tag_spec.albumart_size = (unsigned int)size;
         gfc->tag_spec.albumart_mimetype = mimetype;
         gfc->tag_spec.flags |= CHANGED_FLAG;
         id3tag_add_v2(gfp);
@@ -658,9 +798,9 @@ setLang(char *dst, char const *src)
 {
     int     i;
     if (src == 0 || src[0] == 0) {
-        dst[0] = 'X';
-        dst[1] = 'X';
-        dst[2] = 'X';
+        dst[0] = 'e';
+        dst[1] = 'n';
+        dst[2] = 'g';
     }
     else {
         for (i = 0; i < 3 && src && *src; ++i) {
@@ -723,12 +863,13 @@ isSameDescriptorUcs2(FrameDataNode const *node, unsigned short const *dsc)
 }
 
 static int
-id3v2_add_ucs2(lame_internal_flags * gfc, uint32_t frame_id, char const *lang,
-               unsigned short const *desc, unsigned short const *text)
+id3v2_add_ucs2(lame_t gfp, uint32_t frame_id, char const *lng, unsigned short const *desc, unsigned short const *text)
 {
+    lame_internal_flags *gfc = gfp != 0 ? gfp->internal_flags : 0;
     if (gfc != 0) {
-        FrameDataNode *node = 0;
-        node = findNode(&gfc->tag_spec, frame_id, 0);
+        FrameDataNode *node = findNode(&gfc->tag_spec, frame_id, 0);
+        char lang[4];
+        setLang(lang, lng);
         if (isMultiFrame(frame_id)) {
             while (node) {
                 if (isSameLang(node->lng, lang)) {
@@ -740,7 +881,7 @@ id3v2_add_ucs2(lame_internal_flags * gfc, uint32_t frame_id, char const *lang,
             }
         }
         if (node == 0) {
-            node = calloc(1, sizeof(FrameDataNode));
+            node = lame_calloc(FrameDataNode, 1);
             if (node == 0) {
                 return -254; /* memory problem */
             }
@@ -753,17 +894,19 @@ id3v2_add_ucs2(lame_internal_flags * gfc, uint32_t frame_id, char const *lang,
         node->txt.dim = local_ucs2_strdup(&node->txt.ptr.u, text);
         node->txt.enc = 1;
         gfc->tag_spec.flags |= (CHANGED_FLAG | ADD_V2_FLAG);
+        return 0;
     }
-    return 0;
+    return -255;
 }
 
 static int
-id3v2_add_latin1(lame_internal_flags * gfc, uint32_t frame_id, char const *lang, char const *desc,
-                 char const *text)
+id3v2_add_latin1(lame_t gfp, uint32_t frame_id, char const *lng, char const *desc, char const *text)
 {
+    lame_internal_flags *gfc = gfp != 0 ? gfp->internal_flags : 0;
     if (gfc != 0) {
-        FrameDataNode *node = 0;
-        node = findNode(&gfc->tag_spec, frame_id, 0);
+        FrameDataNode *node = findNode(&gfc->tag_spec, frame_id, 0);
+        char lang[4];
+        setLang(lang, lng);
         if (isMultiFrame(frame_id)) {
             while (node) {
                 if (isSameLang(node->lng, lang)) {
@@ -775,7 +918,7 @@ id3v2_add_latin1(lame_internal_flags * gfc, uint32_t frame_id, char const *lang,
             }
         }
         if (node == 0) {
-            node = calloc(1, sizeof(FrameDataNode));
+            node = lame_calloc(FrameDataNode, 1);
             if (node == 0) {
                 return -254; /* memory problem */
             }
@@ -788,42 +931,61 @@ id3v2_add_latin1(lame_internal_flags * gfc, uint32_t frame_id, char const *lang,
         node->txt.dim = local_strdup(&node->txt.ptr.l, text);
         node->txt.enc = 0;
         gfc->tag_spec.flags |= (CHANGED_FLAG | ADD_V2_FLAG);
+        return 0;
     }
+    return -255;
+}
+
+static char const*
+id3v2_get_language(lame_t gfp)
+{
+    lame_internal_flags const* gfc = gfp ? gfp->internal_flags : 0;
+    if (gfc) return gfc->tag_spec.language;
     return 0;
 }
 
+static int
+id3v2_add_ucs2_lng(lame_t gfp, uint32_t frame_id, unsigned short const *desc, unsigned short const *text)
+{
+    char const* lang = id3v2_get_language(gfp);
+    return id3v2_add_ucs2(gfp, frame_id, lang, desc, text);
+}
 
 static int
-id3tag_set_userinfo_latin1(lame_internal_flags* gfc, uint32_t id, char const *fieldvalue)
+id3v2_add_latin1_lng(lame_t gfp, uint32_t frame_id, char const *desc, char const *text)
 {
-    int     rc;
-    char* dsc = 0, *val;
-    local_strdup(&dsc, fieldvalue);
-    val = dsc;
-    while (*val) {
-        if (*val == '=') {
-            *val++ = 0;
-            break;
-        }
-        ++val;
+    char const* lang = id3v2_get_language(gfp);
+    return id3v2_add_latin1(gfp, frame_id, lang, desc, text);
+}
+
+static int
+id3tag_set_userinfo_latin1(lame_t gfp, uint32_t id, char const *fieldvalue)
+{
+    char const separator = '=';
+    int     rc = -7;
+    int     a = local_char_pos(fieldvalue, separator);
+    if (a >= 0) {
+        char*   dup = 0;
+        local_strdup(&dup, fieldvalue);
+        dup[a] = 0;
+        rc = id3v2_add_latin1_lng(gfp, id, dup, dup+a+1);
+        free(dup);
     }
-    rc = id3v2_add_latin1(gfc, id, "XXX", dsc, val);
-    free(dsc);
     return rc;
 }
 
 static int
-id3tag_set_userinfo_ucs2(lame_internal_flags* gfc, uint32_t id, unsigned short const *fieldvalue)
+id3tag_set_userinfo_ucs2(lame_t gfp, uint32_t id, unsigned short const *fieldvalue)
 {
-    int     a, b, rc = -7;
     unsigned short const separator = fromLatin1Char(fieldvalue,'=');
-    b = local_ucs2_strlen(fieldvalue);
-    a = local_ucs2_pos(fieldvalue, separator);
-    if (a >= 0 && a <= b) { 
+    int     rc = -7;
+    size_t  b = local_ucs2_strlen(fieldvalue);
+    int     a = local_ucs2_pos(fieldvalue, separator);
+    if (a >= 0) {
         unsigned short* dsc = 0, *val = 0;
         local_ucs2_substr(&dsc, fieldvalue, 0, a);
         local_ucs2_substr(&val, fieldvalue, a+1, b);
-        rc = id3v2_add_ucs2(gfc, id, "XXX", dsc, val);
+        rc = id3v2_add_ucs2_lng(gfp, id, dsc, val);
         free(dsc);
         free(val);
     }
@@ -831,12 +993,36 @@ id3tag_set_userinfo_ucs2(lame_internal_flags* gfc, uint32_t id, unsigned short c
 }
 
 int
-id3tag_set_textinfo_utf16(lame_global_flags * gfp, char const *id, unsigned short const *text)
+id3tag_set_textinfo_utf16(lame_t gfp, char const *id, unsigned short const *text)
 {
     uint32_t const frame_id = toID3v2TagId(id);
     if (frame_id == 0) {
         return -1;
     }
+    if (is_lame_internal_flags_null(gfp)) {
+        return 0;
+    }
+    if (text == 0) {
+        return 0;
+    }
+    if (!hasUcs2ByteOrderMarker(text[0])) {
+        return -3;  /* BOM missing */
+    }
+    if (frame_id == ID_TXXX || frame_id == ID_WXXX || frame_id == ID_COMMENT) {
+        return id3tag_set_userinfo_ucs2(gfp, frame_id, text);
+    }
+    if (frame_id == ID_GENRE) {
+        return id3tag_set_genre_utf16(gfp, text);
+    }
+    if (frame_id == ID_PCST) {
+        return id3v2_add_ucs2_lng(gfp, frame_id, 0, text);
+    }
+    if (frame_id == ID_USER) {
+        return id3v2_add_ucs2_lng(gfp, frame_id, text, 0);
+    }
+    if (frame_id == ID_WFED) {
+        return id3v2_add_ucs2_lng(gfp, frame_id, text, 0); /* iTunes expects WFED to be a text frame */
+    }
     if (isFrameIdMatching(frame_id, FRAME_ID('T', 0, 0, 0))
       ||isFrameIdMatching(frame_id, FRAME_ID('W', 0, 0, 0))) {
 #if 0
@@ -844,126 +1030,127 @@ id3tag_set_textinfo_utf16(lame_global_flags * gfp, char const *id, unsigned shor
             return -2;  /* must be Latin-1 encoded */
         }
 #endif
-        if (text == 0) {
-            return 0;
-        }
-        if (!hasUcs2ByteOrderMarker(text[0])) {
-            return -3;  /* BOM missing */
-        }
-        if (gfp != 0) {
-            if (frame_id == ID_TXXX || frame_id == ID_WXXX) {
-                return id3tag_set_userinfo_ucs2(gfp->internal_flags, frame_id, text);
-            }
-            return id3v2_add_ucs2(gfp->internal_flags, frame_id, 0, 0, text);
-        }
+        return id3v2_add_ucs2_lng(gfp, frame_id, 0, text);
     }
     return -255;        /* not supported by now */
 }
 
 extern int
-id3tag_set_textinfo_ucs2(lame_global_flags * gfp, char const *id, unsigned short const *text);
+id3tag_set_textinfo_ucs2(lame_t gfp, char const *id, unsigned short const *text);
 
 int
-id3tag_set_textinfo_ucs2(lame_global_flags * gfp, char const *id, unsigned short const *text)
+id3tag_set_textinfo_ucs2(lame_t gfp, char const *id, unsigned short const *text)
 {
     return id3tag_set_textinfo_utf16(gfp, id, text);
 }
 
 int
-id3tag_set_textinfo_latin1(lame_global_flags * gfp, char const *id, char const *text)
+id3tag_set_textinfo_latin1(lame_t gfp, char const *id, char const *text)
 {
     uint32_t const frame_id = toID3v2TagId(id);
     if (frame_id == 0) {
         return -1;
     }
+    if (is_lame_internal_flags_null(gfp)) {
+        return 0;
+    }
+    if (text == 0) {
+        return 0;
+    }
+    if (frame_id == ID_TXXX || frame_id == ID_WXXX || frame_id == ID_COMMENT) {
+        return id3tag_set_userinfo_latin1(gfp, frame_id, text);
+    }
+    if (frame_id == ID_GENRE) {
+        return id3tag_set_genre(gfp, text);
+    }
+    if (frame_id == ID_PCST) {
+        return id3v2_add_latin1_lng(gfp, frame_id, 0, text);
+    }
+    if (frame_id == ID_USER) {
+        return id3v2_add_latin1_lng(gfp, frame_id, text, 0);
+    }
+    if (frame_id == ID_WFED) {
+        return id3v2_add_latin1_lng(gfp, frame_id, text, 0); /* iTunes expects WFED to be a text frame */
+    }
     if (isFrameIdMatching(frame_id, FRAME_ID('T', 0, 0, 0))
       ||isFrameIdMatching(frame_id, FRAME_ID('W', 0, 0, 0))) {
-        if (text == 0) {
-            return 0;
-        }
-        if (gfp != 0) {
-            if (frame_id == ID_TXXX || frame_id == ID_WXXX) {
-                return id3tag_set_userinfo_latin1(gfp->internal_flags, frame_id, text);
-            }
-            return id3v2_add_latin1(gfp->internal_flags, frame_id, 0, 0, text);
-        }
+        return id3v2_add_latin1_lng(gfp, frame_id, 0, text);
     }
     return -255;        /* not supported by now */
 }
 
 
 int
-id3tag_set_comment_latin1(lame_global_flags * gfp, char const *lang, char const *desc,
-                          char const *text)
+id3tag_set_comment_latin1(lame_t gfp, char const *lang, char const *desc, char const *text)
 {
-    if (gfp != 0) {
-        return id3v2_add_latin1(gfp->internal_flags, ID_COMMENT, lang, desc, text);
+    if (is_lame_internal_flags_null(gfp)) {
+        return 0;
     }
-    return -255;
+    return id3v2_add_latin1(gfp, ID_COMMENT, lang, desc, text);
 }
 
 
 int
-id3tag_set_comment_utf16(lame_global_flags * gfp, char const *lang, unsigned short const *desc,
-                         unsigned short const *text)
+id3tag_set_comment_utf16(lame_t gfp, char const *lang, unsigned short const *desc, unsigned short const *text)
 {
-    if (gfp != 0) {
-        return id3v2_add_ucs2(gfp->internal_flags, ID_COMMENT, lang, desc, text);
+    if (is_lame_internal_flags_null(gfp)) {
+        return 0;
     }
-    return -255;
+    return id3v2_add_ucs2(gfp, ID_COMMENT, lang, desc, text);
 }
 
 extern int
-id3tag_set_comment_ucs2(lame_global_flags * gfp, char const *lang, unsigned short const *desc,
-                        unsigned short const *text);
+id3tag_set_comment_ucs2(lame_t gfp, char const *lang, unsigned short const *desc, unsigned short const *text);
 
 
 int
-id3tag_set_comment_ucs2(lame_global_flags * gfp, char const *lang, unsigned short const *desc,
-                        unsigned short const *text)
+id3tag_set_comment_ucs2(lame_t gfp, char const *lang, unsigned short const *desc, unsigned short const *text)
 {
+    if (is_lame_internal_flags_null(gfp)) {
+        return 0;
+    }
     return id3tag_set_comment_utf16(gfp, lang, desc, text);
 }
 
 
 void
-id3tag_set_title(lame_global_flags * gfp, const char *title)
+id3tag_set_title(lame_t gfp, const char *title)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
-    if (title && *title) {
+    lame_internal_flags *gfc = gfp != 0 ? gfp->internal_flags : 0;
+    if (gfc && title && *title) {
         local_strdup(&gfc->tag_spec.title, title);
         gfc->tag_spec.flags |= CHANGED_FLAG;
-        copyV1ToV2(gfc, ID_TITLE, title);
+        copyV1ToV2(gfp, ID_TITLE, title);
     }
 }
 
 void
-id3tag_set_artist(lame_global_flags * gfp, const char *artist)
+id3tag_set_artist(lame_t gfp, const char *artist)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
-    if (artist && *artist) {
+    lame_internal_flags *gfc = gfp != 0 ? gfp->internal_flags : 0;
+    if (gfc && artist && *artist) {
         local_strdup(&gfc->tag_spec.artist, artist);
         gfc->tag_spec.flags |= CHANGED_FLAG;
-        copyV1ToV2(gfc, ID_ARTIST, artist);
+        copyV1ToV2(gfp, ID_ARTIST, artist);
     }
 }
 
 void
-id3tag_set_album(lame_global_flags * gfp, const char *album)
+id3tag_set_album(lame_t gfp, const char *album)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
-    if (album && *album) {
+    lame_internal_flags *gfc = gfp != 0 ? gfp->internal_flags : 0;
+    if (gfc && album && *album) {
         local_strdup(&gfc->tag_spec.album, album);
         gfc->tag_spec.flags |= CHANGED_FLAG;
-        copyV1ToV2(gfc, ID_ALBUM, album);
+        copyV1ToV2(gfp, ID_ALBUM, album);
     }
 }
 
 void
-id3tag_set_year(lame_global_flags * gfp, const char *year)
+id3tag_set_year(lame_t gfp, const char *year)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
-    if (year && *year) {
+    lame_internal_flags *gfc = gfp != 0 ? gfp->internal_flags : 0;
+    if (gfc && year && *year) {
         int     num = atoi(year);
         if (num < 0) {
             num = 0;
@@ -976,33 +1163,33 @@ id3tag_set_year(lame_global_flags * gfp, const char *year)
             gfc->tag_spec.year = num;
             gfc->tag_spec.flags |= CHANGED_FLAG;
         }
-        copyV1ToV2(gfc, ID_YEAR, year);
+        copyV1ToV2(gfp, ID_YEAR, year);
     }
 }
 
 void
-id3tag_set_comment(lame_global_flags * gfp, const char *comment)
+id3tag_set_comment(lame_t gfp, const char *comment)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
-    if (comment && *comment) {
+    lame_internal_flags *gfc = gfp != 0 ? gfp->internal_flags : 0;
+    if (gfc && comment && *comment) {
         local_strdup(&gfc->tag_spec.comment, comment);
         gfc->tag_spec.flags |= CHANGED_FLAG;
         {
             uint32_t const flags = gfc->tag_spec.flags;
-            id3v2_add_latin1(gfc, ID_COMMENT, "XXX", "", comment);
+            id3v2_add_latin1_lng(gfp, ID_COMMENT, "", comment);
             gfc->tag_spec.flags = flags;
         }
     }
 }
 
 int
-id3tag_set_track(lame_global_flags * gfp, const char *track)
+id3tag_set_track(lame_t gfp, const char *track)
 {
     char const *trackcount;
-    lame_internal_flags *gfc = gfp->internal_flags;
+    lame_internal_flags *gfc = gfp != 0 ? gfp->internal_flags : 0;
     int     ret = 0;
 
-    if (track && *track) {
+    if (gfc && track && *track) {
         int     num = atoi(track);
         /* check for valid ID3v1 track number range */
         if (num < 1 || num > 255) {
@@ -1019,7 +1206,7 @@ id3tag_set_track(lame_global_flags * gfp, const char *track)
         if (trackcount && *trackcount) {
             gfc->tag_spec.flags |= (CHANGED_FLAG | ADD_V2_FLAG);
         }
-        copyV1ToV2(gfc, ID_TRACK, track);
+        copyV1ToV2(gfp, ID_TRACK, track);
     }
     return ret;
 }
@@ -1110,67 +1297,28 @@ searchGenre(const char* genre)
 
 
 int
-id3tag_set_genre(lame_global_flags * gfp, const char *genre)
+id3tag_set_genre(lame_t gfp, const char *genre)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
+    lame_internal_flags *gfc = gfp != 0 ? gfp->internal_flags : 0;
     int     ret = 0;
-    if (genre && *genre) {
-        char   *str;
-        int     num = strtol(genre, &str, 10);
-        /* is the input a string or a valid number? */
-        if (*str) {
-            num = searchGenre(genre);
-            if (num == GENRE_NAME_COUNT) {
-                num = sloppySearchGenre(genre);
-            }
-            if (num == GENRE_NAME_COUNT) {
-                num = GENRE_INDEX_OTHER;
-                ret = -2;
-            }
-            else {
-                genre = genre_names[num];
-            }
-        }
-        else {
-            if ((num < 0) || (num >= GENRE_NAME_COUNT)) {
-                return -1;
-            }
+    if (gfc && genre && *genre) {
+        int const num = lookupGenre(genre);
+        if (num == -1) return num;
+        gfc->tag_spec.flags |= CHANGED_FLAG;
+        if (num >= 0) {
+            gfc->tag_spec.genre_id3v1 = num;
             genre = genre_names[num];
         }
-        gfc->tag_spec.genre_id3v1 = num;
-        gfc->tag_spec.flags |= CHANGED_FLAG;
-        if (ret) {
+        else {
+            gfc->tag_spec.genre_id3v1 = GENRE_INDEX_OTHER;
             gfc->tag_spec.flags |= ADD_V2_FLAG;
         }
-        copyV1ToV2(gfc, ID_GENRE, genre);
+        copyV1ToV2(gfp, ID_GENRE, genre);
     }
     return ret;
 }
 
 
-static unsigned char *
-set_frame_custom(unsigned char *frame, const char *fieldvalue)
-{
-    if (fieldvalue && *fieldvalue) {
-        const char *value = fieldvalue + 5;
-        size_t  length = strlen(value);
-        *frame++ = *fieldvalue++;
-        *frame++ = *fieldvalue++;
-        *frame++ = *fieldvalue++;
-        *frame++ = *fieldvalue++;
-        frame = set_4_byte_value(frame, (unsigned long) (strlen(value) + 1));
-        /* clear 2-byte header flags */
-        *frame++ = 0;
-        *frame++ = 0;
-        /* clear 1 encoding descriptor byte to indicate ISO-8859-1 format */
-        *frame++ = 0;
-        while (length--) {
-            *frame++ = *value++;
-        }
-    }
-    return frame;
-}
-
 static  size_t
 sizeOfNode(FrameDataNode const *node)
 {
@@ -1245,14 +1393,16 @@ sizeOfWxxxNode(FrameDataNode const *node)
                 break;
             }
         }
-        switch (node->txt.enc) {
-        default:
-        case 0:
-            n += node->txt.dim;
-            break;
-        case 1:
-            n += node->txt.dim - 1; /* UCS2 -> Latin1, skip BOM */
-            break;
+        if (node->txt.dim > 0) {
+            switch (node->txt.enc) {
+            default:
+            case 0:
+                n += node->txt.dim;
+                break;
+            case 1:
+                n += node->txt.dim - 1; /* UCS2 -> Latin1, skip BOM */
+                break;
+            }
         }
     }
     return n;
@@ -1384,7 +1534,7 @@ set_frame_wxxx(unsigned char *frame, FrameDataNode const *node)
         *frame++ = 0;
         if (node->dsc.dim > 0) {
             /* clear 1 encoding descriptor byte to indicate ISO-8859-1 format */
-            *frame++ = node->txt.enc == 1 ? 1 : 0;
+            *frame++ = node->dsc.enc == 1 ? 1 : 0;
             if (node->dsc.enc != 1) {
                 frame = writeChars(frame, node->dsc.ptr.l, node->dsc.dim);
                 *frame++ = 0;
@@ -1442,35 +1592,26 @@ set_frame_apic(unsigned char *frame, const char *mimetype, const unsigned char *
 }
 
 int
-id3tag_set_fieldvalue(lame_global_flags * gfp, const char *fieldvalue)
+id3tag_set_fieldvalue(lame_t gfp, const char *fieldvalue)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
+    if (is_lame_internal_flags_null(gfp)) {
+        return 0;
+    }
     if (fieldvalue && *fieldvalue) {
-        uint32_t const frame_id = toID3v2TagId(fieldvalue);
-        char  **p = NULL;
         if (strlen(fieldvalue) < 5 || fieldvalue[4] != '=') {
             return -1;
         }
-        if (frame_id != 0) {
-            if (id3tag_set_textinfo_latin1(gfp, fieldvalue, &fieldvalue[5])) {
-                p = (char **) realloc(gfc->tag_spec.values,
-                                      sizeof(char *) * (gfc->tag_spec.num_values + 1));
-                if (!p) {
-                    return -1;
-                }
-                gfc->tag_spec.values = (char **) p;
-                local_strdup(&gfc->tag_spec.values[gfc->tag_spec.num_values++], fieldvalue);
-            }
-        }
-        gfc->tag_spec.flags |= CHANGED_FLAG;
+        return id3tag_set_textinfo_latin1(gfp, fieldvalue, &fieldvalue[5]);
     }
-    id3tag_add_v2(gfp);
     return 0;
 }
 
 int
-id3tag_set_fieldvalue_utf16(lame_global_flags * gfp, const unsigned short *fieldvalue)
+id3tag_set_fieldvalue_utf16(lame_t gfp, const unsigned short *fieldvalue)
 {
+    if (is_lame_internal_flags_null(gfp)) {
+        return 0;
+    }
     if (fieldvalue && *fieldvalue) {
         size_t dx = hasUcs2ByteOrderMarker(fieldvalue[0]);
         unsigned short const separator = fromLatin1Char(fieldvalue, '=');
@@ -1487,7 +1628,7 @@ id3tag_set_fieldvalue_utf16(lame_global_flags * gfp, const unsigned short *field
             unsigned short* txt = 0;
             int     rc;
             local_ucs2_substr(&txt, fieldvalue, dx+5, local_ucs2_strlen(fieldvalue));
-            rc = id3tag_set_textinfo_ucs2(gfp, fid, txt);
+            rc = id3tag_set_textinfo_utf16(gfp, fid, txt);
             free(txt);
             return rc;
         }
@@ -1496,25 +1637,26 @@ id3tag_set_fieldvalue_utf16(lame_global_flags * gfp, const unsigned short *field
 }
 
 extern int
-id3tag_set_fieldvalue_ucs2(lame_global_flags * gfp, const unsigned short *fieldvalue);
+id3tag_set_fieldvalue_ucs2(lame_t gfp, const unsigned short *fieldvalue);
 
 int
-id3tag_set_fieldvalue_ucs2(lame_global_flags * gfp, const unsigned short *fieldvalue)
+id3tag_set_fieldvalue_ucs2(lame_t gfp, const unsigned short *fieldvalue)
 {
+    if (is_lame_internal_flags_null(gfp)) {
+        return 0;
+    }
     return id3tag_set_fieldvalue_utf16(gfp, fieldvalue);
 }
 
 size_t
-lame_get_id3v2_tag(lame_global_flags * gfp, unsigned char *buffer, size_t size)
+lame_get_id3v2_tag(lame_t gfp, unsigned char *buffer, size_t size)
 {
-    lame_internal_flags *gfc;
-    if (gfp == 0) {
+    lame_internal_flags *gfc = 0;
+
+    if (is_lame_internal_flags_null(gfp)) {
         return 0;
     }
     gfc = gfp->internal_flags;
-    if (gfc == 0) {
-        return 0;
-    }
     if (test_tag_spec_flags(gfc, V1_ONLY_FLAG)) {
         return 0;
     }
@@ -1540,21 +1682,17 @@ lame_get_id3v2_tag(lame_global_flags * gfp, unsigned char *buffer, size_t size)
             size_t  tag_size;
             unsigned char *p;
             size_t  adjusted_tag_size;
-            unsigned int i;
             const char *albumart_mime = NULL;
             static const char *mime_jpeg = "image/jpeg";
             static const char *mime_png = "image/png";
             static const char *mime_gif = "image/gif";
 
             if (gfp->num_samples != MAX_U_32_NUM) {
-                id3v2AddAudioDuration(gfc, gfp->num_samples);
+                id3v2AddAudioDuration(gfp, gfp->num_samples);
             }
 
             /* calulate size of tag starting with 10-byte tag header */
             tag_size = 10;
-            for (i = 0; i < gfc->tag_spec.num_values; ++i) {
-                tag_size += 6 + strlen(gfc->tag_spec.values[i]);
-            }
             if (gfc->tag_spec.albumart && gfc->tag_spec.albumart_size) {
                 switch (gfc->tag_spec.albumart_mimetype) {
                 case MIMETYPE_JPEG:
@@ -1576,7 +1714,7 @@ lame_get_id3v2_tag(lame_global_flags * gfp, unsigned char *buffer, size_t size)
                 if (tag->v2_head != 0) {
                     FrameDataNode *node;
                     for (node = tag->v2_head; node != 0; node = node->nxt) {
-                        if (node->fid == ID_COMMENT) {
+                        if (node->fid == ID_COMMENT || node->fid == ID_USER) {
                             tag_size += sizeOfCommentNode(node);
                         }
                         else if (isFrameIdMatching(node->fid, FRAME_ID('W',0,0,0))) {
@@ -1633,7 +1771,7 @@ lame_get_id3v2_tag(lame_global_flags * gfp, unsigned char *buffer, size_t size)
                 if (tag->v2_head != 0) {
                     FrameDataNode *node;
                     for (node = tag->v2_head; node != 0; node = node->nxt) {
-                        if (node->fid == ID_COMMENT) {
+                        if (node->fid == ID_COMMENT || node->fid == ID_USER) {
                             p = set_frame_comment(p, node);
                         }
                         else if (isFrameIdMatching(node->fid,FRAME_ID('W',0,0,0))) {
@@ -1645,9 +1783,6 @@ lame_get_id3v2_tag(lame_global_flags * gfp, unsigned char *buffer, size_t size)
                     }
                 }
             }
-            for (i = 0; i < gfc->tag_spec.num_values; ++i) {
-                p = set_frame_custom(p, gfc->tag_spec.values[i]);
-            }
             if (albumart_mime) {
                 p = set_frame_apic(p, albumart_mime, gfc->tag_spec.albumart,
                                    gfc->tag_spec.albumart_size);
@@ -1661,9 +1796,14 @@ lame_get_id3v2_tag(lame_global_flags * gfp, unsigned char *buffer, size_t size)
 }
 
 int
-id3tag_write_v2(lame_global_flags * gfp)
+id3tag_write_v2(lame_t gfp)
 {
-    lame_internal_flags *gfc = gfp->internal_flags;
+    lame_internal_flags *gfc = 0;
+
+    if (is_lame_internal_flags_null(gfp)) {
+        return 0;
+    }
+    gfc = gfp->internal_flags;
 #if 0
     debug_tag_spec_flags(gfc, "write v2");
 #endif
@@ -1675,7 +1815,7 @@ id3tag_write_v2(lame_global_flags * gfp)
         size_t  tag_size, n;
 
         n = lame_get_id3v2_tag(gfp, 0, 0);
-        tag = malloc(n);
+        tag = lame_calloc(unsigned char, n);
         if (tag == 0) {
             return -1;
         }
@@ -1712,7 +1852,7 @@ set_text_field(unsigned char *field, const char *text, size_t size, int pad)
 }
 
 size_t
-lame_get_id3v1_tag(lame_global_flags * gfp, unsigned char *buffer, size_t size)
+lame_get_id3v1_tag(lame_t gfp, unsigned char *buffer, size_t size)
 {
     size_t const tag_size = 128;
     lame_internal_flags *gfc;
@@ -1762,12 +1902,17 @@ lame_get_id3v1_tag(lame_global_flags * gfp, unsigned char *buffer, size_t size)
 }
 
 int
-id3tag_write_v1(lame_global_flags * gfp)
+id3tag_write_v1(lame_t gfp)
 {
-    lame_internal_flags *const gfc = gfp->internal_flags;
+    lame_internal_flags* gfc = 0;
     size_t  i, n, m;
     unsigned char tag[128];
 
+    if (is_lame_internal_flags_null(gfp)) {
+        return 0;
+    }
+    gfc = gfp->internal_flags;
+
     m = sizeof(tag);
     n = lame_get_id3v1_tag(gfp, tag, m);
     if (n > m) {
