@@ -4,7 +4,7 @@
  *
  *      Copyright (c) 1999-2000 Mark Taylor
  *      Copyright (c) 2000-2005 Takehiro Tominaga
- *      Copyright (c) 2000-2011 Robert Hegemann
+ *      Copyright (c) 2000-2017 Robert Hegemann
  *      Copyright (c) 2000-2005 Gabriel Bouvigne
  *      Copyright (c) 2000-2004 Alexander Leidinger
  *
@@ -24,7 +24,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: lame.c,v 1.364 2011/05/07 16:05:17 rbrito Exp $ */
+/* $Id: lame.c,v 1.377 2017/09/26 12:14:02 robert Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -83,6 +83,8 @@ is_lame_internal_flags_valid(const lame_internal_flags * gfc)
         return 0;
     if (gfc->class_id != LAME_ID)
         return 0;
+    if (gfc->lame_init_params_successful <=0)
+        return 0;
     return 1;
 }
 
@@ -384,7 +386,7 @@ lame_init_qval(lame_global_flags * gfp)
         cfg->noise_shaping_stop = 0;
         cfg->use_best_huffman = 0;
         cfg->full_outer_loop = 0;
-        if (gfp->VBR == vbr_mt || gfp->VBR == vbr_mtrh) {
+        if (cfg->vbr == vbr_mt || cfg->vbr == vbr_mtrh) {
             cfg->full_outer_loop  = -1;
         }
         break;
@@ -540,10 +542,34 @@ lame_init_params(lame_global_flags * gfp)
 
     int     i;
     int     j;
-    lame_internal_flags *const gfc = gfp->internal_flags;
-    SessionConfig_t *const cfg = &gfc->cfg;
+    lame_internal_flags *gfc;
+    SessionConfig_t *cfg;
 
-    gfc->class_id = 0;
+    if (!is_lame_global_flags_valid(gfp)) 
+        return -1;
+
+    gfc = gfp->internal_flags;
+    if (gfc == 0) 
+        return -1;
+
+    if (is_lame_internal_flags_valid(gfc))
+        return -1; /* already initialized */
+
+    /* start updating lame internal flags */
+    gfc->class_id = LAME_ID;
+    gfc->lame_init_params_successful = 0; /* will be set to one, when we get through until the end */
+
+    if (gfp->samplerate_in < 1)
+        return -1; /* input sample rate makes no sense */
+    if (gfp->num_channels < 1 || 2 < gfp->num_channels)
+        return -1; /* number of input channels makes no sense */
+    if (gfp->samplerate_out != 0) {
+        int   v=0;
+        if (SmpFrqIndex(gfp->samplerate_out, &v) < 0)
+            return -1; /* output sample rate makes no sense */
+    }
+
+    cfg = &gfc->cfg;
 
     cfg->enforce_min_bitrate = gfp->VBR_hard_min;
     cfg->analysis = gfp->analysis;
@@ -579,20 +605,7 @@ lame_init_params(lame_global_flags * gfp)
     }
 
 
-    if (NULL == gfc->ATH)
-        gfc->ATH = calloc(1, sizeof(ATH_t));
-
-    if (NULL == gfc->ATH)
-        return -2;      /* maybe error codes should be enumerated in lame.h ?? */
-
-    if (NULL == gfc->sv_rpg.rgdata)
-        gfc->sv_rpg.rgdata = calloc(1, sizeof(replaygain_t));
-    if (NULL == gfc->sv_rpg.rgdata) {
-        freegfc(gfc);
-        gfp->internal_flags = NULL;
-        return -2;
-    }
-
+    cfg->vbr = gfp->VBR;
     cfg->error_protection = gfp->error_protection;
     cfg->copyright = gfp->copyright;
     cfg->original = gfp->original;
@@ -603,14 +616,14 @@ lame_init_params(lame_global_flags * gfp)
     if (cfg->channels_in == 1)
         gfp->mode = MONO;
     cfg->channels_out = (gfp->mode == MONO) ? 1 : 2;
-    if (gfp->mode == MONO)
-        gfp->force_ms = 0; /* don't allow forced mid/side stereo for mono output */
+    if (gfp->mode != JOINT_STEREO)
+        gfp->force_ms = 0; /* forced mid/side stereo for j-stereo only */
     cfg->force_ms = gfp->force_ms;
 
-    if (gfp->VBR == vbr_off && gfp->VBR_mean_bitrate_kbps != 128 && gfp->brate == 0)
+    if (cfg->vbr == vbr_off && gfp->VBR_mean_bitrate_kbps != 128 && gfp->brate == 0)
         gfp->brate = gfp->VBR_mean_bitrate_kbps;
 
-    switch (gfp->VBR) {
+    switch (cfg->vbr) {
     case vbr_off:
     case vbr_mtrh:
     case vbr_mt:
@@ -623,14 +636,14 @@ lame_init_params(lame_global_flags * gfp)
 
     cfg->free_format = gfp->free_format;
 
-    if (gfp->VBR == vbr_off && gfp->brate == 0) {
+    if (cfg->vbr == vbr_off && gfp->brate == 0) {
         /* no bitrate or compression ratio specified, use 11.025 */
         if (EQ(gfp->compression_ratio, 0))
             gfp->compression_ratio = 11.025; /* rate to compress a CD down to exactly 128000 bps */
     }
 
     /* find bitrate if user specify a compression ratio */
-    if (gfp->VBR == vbr_off && gfp->compression_ratio > 0) {
+    if (cfg->vbr == vbr_off && gfp->compression_ratio > 0) {
 
         if (gfp->samplerate_out == 0)
             gfp->samplerate_out = map2MP3Frequency((int) (0.97 * gfp->samplerate_in)); /* round up with a margin of 3% */
@@ -642,6 +655,7 @@ lame_init_params(lame_global_flags * gfp)
 
         /* we need the version for the bitrate table look up */
         cfg->samplerate_index = SmpFrqIndex(gfp->samplerate_out, &cfg->version);
+        assert(cfg->samplerate_index >=0);
 
         if (!cfg->free_format) /* for non Free Format find the nearest allowed bitrate */
             gfp->brate = FindNearestBitrate(gfp->brate, cfg->version, gfp->samplerate_out);
@@ -662,7 +676,7 @@ lame_init_params(lame_global_flags * gfp)
     }
     /* WORK IN PROGRESS */
     /* mapping VBR scale to internal VBR quality settings */
-    if (gfp->samplerate_out == 0 && (gfp->VBR == vbr_mt || gfp->VBR == vbr_mtrh)) {
+    if (gfp->samplerate_out == 0 && (cfg->vbr == vbr_mt || cfg->vbr == vbr_mtrh)) {
         float const qval = gfp->VBR_q + gfp->VBR_q_frac;
         struct q_map { int sr_a; float qa, qb, ta, tb; int lp; };
         struct q_map const m[9]
@@ -709,7 +723,7 @@ lame_init_params(lame_global_flags * gfp)
         double  lowpass = 16000;
         double  highpass;
 
-        switch (gfp->VBR) {
+        switch (cfg->vbr) {
         case vbr_off:{
                 optimum_bandwidth(&lowpass, &highpass, gfp->brate);
                 break;
@@ -759,7 +773,7 @@ lame_init_params(lame_global_flags * gfp)
             }
         }
 
-        if (gfp->mode == MONO && (gfp->VBR == vbr_off || gfp->VBR == vbr_abr))
+        if (gfp->mode == MONO && (cfg->vbr == vbr_off || cfg->vbr == vbr_abr))
             lowpass *= 1.5;
 
         gfp->lowpassfreq = lowpass;
@@ -771,7 +785,7 @@ lame_init_params(lame_global_flags * gfp)
         }
         gfp->samplerate_out = optimum_samplefreq((int) gfp->lowpassfreq, gfp->samplerate_in);
     }
-    if (gfp->VBR == vbr_mt || gfp->VBR == vbr_mtrh) {
+    if (cfg->vbr == vbr_mt || cfg->vbr == vbr_mtrh) {
         gfp->lowpassfreq = Min(24000, gfp->lowpassfreq);
     }
     else {
@@ -779,56 +793,20 @@ lame_init_params(lame_global_flags * gfp)
     }
     gfp->lowpassfreq = Min(gfp->samplerate_out / 2, gfp->lowpassfreq);
 
-    if (gfp->VBR == vbr_off) {
+    if (cfg->vbr == vbr_off) {
         gfp->compression_ratio = gfp->samplerate_out * 16 * cfg->channels_out / (1.e3 * gfp->brate);
     }
-    if (gfp->VBR == vbr_abr) {
+    if (cfg->vbr == vbr_abr) {
         gfp->compression_ratio =
             gfp->samplerate_out * 16 * cfg->channels_out / (1.e3 * gfp->VBR_mean_bitrate_kbps);
     }
 
-    /* do not compute ReplayGain values and do not find the peak sample
-       if we can't store them */
-    if (!gfp->write_lame_tag) {
-        gfp->findReplayGain = 0;
-        gfp->decode_on_the_fly = 0;
-        cfg->findPeakSample = 0;
-    }
-
-    cfg->findReplayGain = gfp->findReplayGain;
-    cfg->decode_on_the_fly = gfp->decode_on_the_fly;
-
-    if (cfg->decode_on_the_fly)
-        cfg->findPeakSample = 1;
-
-    if (cfg->findReplayGain) {
-        if (InitGainAnalysis(gfc->sv_rpg.rgdata, gfp->samplerate_out) == INIT_GAIN_ANALYSIS_ERROR) {
-            freegfc(gfc);
-            gfp->internal_flags = NULL;
-            return -6;
-        }
-    }
-
-#ifdef DECODE_ON_THE_FLY
-    if (cfg->decode_on_the_fly && !gfp->decode_only) {
-        if (gfc->hip) {
-            hip_decode_exit(gfc->hip);
-        }
-        gfc->hip = hip_decode_init();
-        /* report functions */
-        hip_set_errorf(gfc->hip, gfp->report.errorf);
-        hip_set_debugf(gfc->hip, gfp->report.debugf);
-        hip_set_msgf(gfc->hip, gfp->report.msgf);
-    }
-#endif
-
     cfg->disable_reservoir = gfp->disable_reservoir;
     cfg->lowpassfreq = gfp->lowpassfreq;
     cfg->highpassfreq = gfp->highpassfreq;
     cfg->samplerate_in = gfp->samplerate_in;
     cfg->samplerate_out = gfp->samplerate_out;
     cfg->mode_gr = cfg->samplerate_out <= 24000 ? 1 : 2; /* Number of granules per frame */
-    gfc->ov_enc.encoder_delay = ENCDELAY;
 
 
     /*
@@ -859,7 +837,7 @@ lame_init_params(lame_global_flags * gfp)
      *  for lower bitrates, downsample with --resample
      */
 
-    switch (gfp->VBR) {
+    switch (cfg->vbr) {
     case vbr_mt:
     case vbr_rh:
     case vbr_mtrh:
@@ -936,13 +914,9 @@ lame_init_params(lame_global_flags * gfp)
    * samplerate and bitrate index
    *******************************************************/
     cfg->samplerate_index = SmpFrqIndex(cfg->samplerate_out, &cfg->version);
-    if (cfg->samplerate_index < 0) {
-        freegfc(gfc);
-        gfp->internal_flags = NULL;
-        return -1;
-    }
+    assert(cfg->samplerate_index >= 0);
 
-    if (gfp->VBR == vbr_off) {
+    if (cfg->vbr == vbr_off) {
         if (cfg->free_format) {
             gfc->ov_enc.bitrate_index = 0;
         }
@@ -950,9 +924,11 @@ lame_init_params(lame_global_flags * gfp)
             gfp->brate = FindNearestBitrate(gfp->brate, cfg->version, cfg->samplerate_out);
             gfc->ov_enc.bitrate_index = BitrateIndex(gfp->brate, cfg->version, cfg->samplerate_out);
             if (gfc->ov_enc.bitrate_index <= 0) {
-                freegfc(gfc);
-                gfp->internal_flags = NULL;
-                return -1;
+                /* This never happens, because of preceding FindNearestBitrate!
+                 * But, set a sane value, just in case
+                 */
+                assert(0);
+                gfc->ov_enc.bitrate_index = 8;
             }
         }
     }
@@ -992,8 +968,6 @@ lame_init_params(lame_global_flags * gfp)
     if (cfg->error_protection)
         cfg->sideinfo_len += 2;
 
-    gfc->class_id = LAME_ID;
-
     {
         int     k;
 
@@ -1007,7 +981,7 @@ lame_init_params(lame_global_flags * gfp)
     assert(gfp->VBR_q <= 9);
     assert(gfp->VBR_q >= 0);
 
-    switch (gfp->VBR) {
+    switch (cfg->vbr) {
 
     case vbr_mt:
     case vbr_mtrh:{
@@ -1038,7 +1012,6 @@ lame_init_params(lame_global_flags * gfp)
             else
                 gfc->sv_qnt.sfb21_extra = (cfg->samplerate_out > 44000);
 
-            gfc->iteration_loop = VBR_new_iteration_loop;
             break;
 
         }
@@ -1065,12 +1038,10 @@ lame_init_params(lame_global_flags * gfp)
             if (gfp->quality < 0)
                 gfp->quality = LAME_DEFAULT_QUALITY;
 
-            gfc->iteration_loop = VBR_old_iteration_loop;
             break;
         }
 
     default:           /* cbr/abr */  {
-            vbr_mode vbrmode;
 
             /*  no sfb21 extra with CBR code
              */
@@ -1080,19 +1051,12 @@ lame_init_params(lame_global_flags * gfp)
                 gfp->quality = LAME_DEFAULT_QUALITY;
 
 
-            vbrmode = gfp->VBR;
-            if (vbrmode == vbr_off)
+            if (cfg->vbr == vbr_off)
                 (void) lame_set_VBR_mean_bitrate_kbps(gfp, gfp->brate);
             /* second, set parameters depending on bitrate */
             (void) apply_preset(gfp, gfp->VBR_mean_bitrate_kbps, 0);
-            gfp->VBR = vbrmode;
+            gfp->VBR = cfg->vbr;
 
-            if (vbrmode == vbr_off) {
-                gfc->iteration_loop = CBR_iteration_loop;
-            }
-            else {
-                gfc->iteration_loop = ABR_iteration_loop;
-            }
             break;
         }
     }
@@ -1109,7 +1073,7 @@ lame_init_params(lame_global_flags * gfp)
     }
 
 
-    if (gfp->VBR != vbr_off) { /* choose a min/max bitrate for VBR */
+    if (cfg->vbr != vbr_off) { /* choose a min/max bitrate for VBR */
         /* if the user didn't specify VBR_max_bitrate: */
         cfg->vbr_min_bitrate_index = 1; /* default: allow   8 kbps (MPEG-2) or  32 kbps (MPEG-1) */
         cfg->vbr_max_bitrate_index = 14; /* default: allow 160 kbps (MPEG-2) or 320 kbps (MPEG-1) */
@@ -1120,16 +1084,26 @@ lame_init_params(lame_global_flags * gfp)
                 FindNearestBitrate(gfp->VBR_min_bitrate_kbps, cfg->version, cfg->samplerate_out);
             cfg->vbr_min_bitrate_index =
                 BitrateIndex(gfp->VBR_min_bitrate_kbps, cfg->version, cfg->samplerate_out);
-            if (cfg->vbr_min_bitrate_index < 0)
-                return -1;
+            if (cfg->vbr_min_bitrate_index < 0) {
+                /* This never happens, because of preceding FindNearestBitrate!
+                 * But, set a sane value, just in case
+                 */
+                assert(0);
+                cfg->vbr_min_bitrate_index = 1;
+            }
         }
         if (gfp->VBR_max_bitrate_kbps) {
             gfp->VBR_max_bitrate_kbps =
                 FindNearestBitrate(gfp->VBR_max_bitrate_kbps, cfg->version, cfg->samplerate_out);
             cfg->vbr_max_bitrate_index =
                 BitrateIndex(gfp->VBR_max_bitrate_kbps, cfg->version, cfg->samplerate_out);
-            if (cfg->vbr_max_bitrate_index < 0)
-                return -1;
+            if (cfg->vbr_max_bitrate_index < 0) {
+                /* This never happens, because of preceding FindNearestBitrate!
+                 * But, set a sane value, just in case
+                 */
+                assert(0);
+                cfg->vbr_max_bitrate_index = cfg->samplerate_out < 16000 ? 8 : 14;
+            }
         }
         gfp->VBR_min_bitrate_kbps = bitrate_table[cfg->version][cfg->vbr_min_bitrate_index];
         gfp->VBR_max_bitrate_kbps = bitrate_table[cfg->version][cfg->vbr_max_bitrate_index];
@@ -1143,7 +1117,6 @@ lame_init_params(lame_global_flags * gfp)
 
     cfg->preset = gfp->preset;
     cfg->write_lame_tag = gfp->write_lame_tag;
-    cfg->vbr = gfp->VBR;
     gfc->sv_qnt.substep_shaping = gfp->substep_shaping;
     cfg->noise_shaping = gfp->noise_shaping;
     cfg->subblock_gain = gfp->subblock_gain;
@@ -1226,7 +1199,12 @@ lame_init_params(lame_global_flags * gfp)
     cfg->quant_comp_short = gfp->quant_comp_short;
 
     cfg->use_temporal_masking_effect = gfp->useTemporal;
-    cfg->use_safe_joint_stereo = gfp->exp_nspsytune & 2;
+    if (cfg->mode == JOINT_STEREO) {
+        cfg->use_safe_joint_stereo = gfp->exp_nspsytune & 2;
+    }
+    else {
+        cfg->use_safe_joint_stereo = 0;
+    }
     {
         cfg->adjust_bass_db = (gfp->exp_nspsytune >> 2) & 63;
         if (cfg->adjust_bass_db >= 32.f)
@@ -1301,9 +1279,48 @@ lame_init_params(lame_global_flags * gfp)
     (void) psymodel_init(gfp);
 
     cfg->buffer_constraint = get_max_frame_buffer_size_by_constraint(cfg, gfp->strict_ISO);
+
+
+    cfg->findReplayGain = gfp->findReplayGain;
+    cfg->decode_on_the_fly = gfp->decode_on_the_fly;
+
+    if (cfg->decode_on_the_fly)
+        cfg->findPeakSample = 1;
+
+    if (cfg->findReplayGain) {
+        if (InitGainAnalysis(gfc->sv_rpg.rgdata, cfg->samplerate_out) == INIT_GAIN_ANALYSIS_ERROR) {
+            /* Actually this never happens, our samplerates are the ones RG accepts!
+             * But just in case, turn RG off
+             */
+            assert(0);
+            cfg->findReplayGain = 0;
+        }
+    }
+
+#ifdef DECODE_ON_THE_FLY
+    if (cfg->decode_on_the_fly && !gfp->decode_only) {
+        if (gfc->hip) {
+            hip_decode_exit(gfc->hip);
+        }
+        gfc->hip = hip_decode_init();
+        /* report functions */
+        hip_set_errorf(gfc->hip, gfp->report.errorf);
+        hip_set_debugf(gfc->hip, gfp->report.debugf);
+        hip_set_msgf(gfc->hip, gfp->report.msgf);
+    }
+#endif
+    /* updating lame internal flags finished successful */
+    gfc->lame_init_params_successful = 1;
     return 0;
 }
 
+static void
+concatSep(char* dest, char const* sep, char const* str)
+{
+    if (*dest != 0) strcat(dest, sep);
+    strcat(dest, str);
+}
+
 /*
  *  print_config
  *
@@ -1328,6 +1345,7 @@ lame_print_config(const lame_global_flags * gfp)
 #endif
     if (gfc->CPU_features.MMX
         || gfc->CPU_features.AMD_3DNow || gfc->CPU_features.SSE || gfc->CPU_features.SSE2) {
+        char    text[256] = { 0 };
         int     fft_asm_used = 0;
 #ifdef HAVE_NASM
         if (gfc->CPU_features.AMD_3DNow) {
@@ -1336,44 +1354,34 @@ lame_print_config(const lame_global_flags * gfp)
         else if (gfc->CPU_features.SSE) {
             fft_asm_used = 2;
         }
-        else
-#endif
+#else
+# if defined( HAVE_XMMINTRIN_H ) && defined( MIN_ARCH_SSE )
         {
-            fft_asm_used = 0;
+            fft_asm_used = 3;
         }
-        MSGF(gfc, "CPU features: ");
-
+# endif
+#endif
         if (gfc->CPU_features.MMX) {
 #ifdef MMX_choose_table
-            MSGF(gfc, "MMX (ASM used)");
+            concatSep(text, ", ", "MMX (ASM used)");
 #else
-            MSGF(gfc, "MMX");
+            concatSep(text, ", ", "MMX");
 #endif
         }
         if (gfc->CPU_features.AMD_3DNow) {
-            if (fft_asm_used == 1) {
-                MSGF(gfc, ", 3DNow! (ASM used)");
-            }
-            else {
-                MSGF(gfc, ", 3DNow!");
-            }
+            concatSep(text, ", ", (fft_asm_used == 1) ? "3DNow! (ASM used)" : "3DNow!");
         }
         if (gfc->CPU_features.SSE) {
 #if defined(HAVE_XMMINTRIN_H)
-            MSGF(gfc, ", SSE (ASM used)");
+            concatSep(text, ", ", "SSE (ASM used)");
 #else
-            if (fft_asm_used == 2) {
-                MSGF(gfc, ", SSE (ASM used)");
-            }
-            else {
-                MSGF(gfc, ", SSE");
-            }
+            concatSep(text, ", ", (fft_asm_used == 2) ? "SSE (ASM used)" : "SSE");
 #endif
         }
         if (gfc->CPU_features.SSE2) {
-            MSGF(gfc, ", SSE2");
+            concatSep(text, ", ", (fft_asm_used == 3) ? "SSE2 (ASM used)" : "SSE2");
         }
-        MSGF(gfc, "\n");
+        MSGF(gfc, "CPU features: %s\n", text);
     }
 
     if (cfg->channels_in == 2 && cfg->channels_out == 1 /* mono */ ) {
@@ -1628,8 +1636,8 @@ update_inbuffer_size(lame_internal_flags * gfc, const int nsamples)
         if (esv->in_buffer_1) {
             free(esv->in_buffer_1);
         }
-        esv->in_buffer_0 = calloc(nsamples, sizeof(sample_t));
-        esv->in_buffer_1 = calloc(nsamples, sizeof(sample_t));
+        esv->in_buffer_0 = lame_calloc(sample_t, nsamples);
+        esv->in_buffer_1 = lame_calloc(sample_t, nsamples);
         esv->in_buffer_nsamples = nsamples;
     }
     if (esv->in_buffer_0 == NULL || esv->in_buffer_1 == NULL) {
@@ -1712,7 +1720,10 @@ lame_encode_buffer_sample_t(lame_internal_flags * gfc,
         return 0;
 
     /* copy out any tags that may have been written into bitstream */
-    mp3out = copy_buffer(gfc, mp3buf, mp3buf_size, 0);
+    {   /* if user specifed buffer size = 0, dont check size */
+        int const buf_size = mp3buf_size == 0 ? INT_MAX : mp3buf_size;
+        mp3out = copy_buffer(gfc, mp3buf, buf_size, 0);
+    }
     if (mp3out < 0)
         return mp3out;  /* not enough buffer space */
     mp3buf += mp3out;
@@ -1777,7 +1788,7 @@ lame_encode_buffer_sample_t(lame_internal_flags * gfc,
 
             int     buf_size = mp3buf_size - mp3size;
             if (mp3buf_size == 0)
-                buf_size = 0;
+                buf_size = INT_MAX;
 
             ret = lame_encode_mp3_frame(gfc, mfbuf[0], mfbuf[1], mp3buf, buf_size);
 
@@ -1998,6 +2009,17 @@ lame_encode_buffer_interleaved(lame_global_flags * gfp,
 }
 
 
+int
+lame_encode_buffer_interleaved_int(lame_t gfp,
+                                   const int pcm[], const int nsamples,
+                                   unsigned char *mp3buf, const int mp3buf_size)
+{
+    /* input is assumed to be normalized to +/- MAX(int) for full scale */
+    FLOAT const norm = (1.0 / (1L << (8 * sizeof(int)-16)));
+    return lame_encode_buffer_template(gfp, pcm, pcm + 1, nsamples, mp3buf, mp3buf_size, pcm_int_type, 2, norm);
+}
+
+
 
 
 /*****************************************************************
@@ -2017,6 +2039,9 @@ lame_encode_flush_nogap(lame_global_flags * gfp, unsigned char *mp3buffer, int m
         lame_internal_flags *const gfc = gfp->internal_flags;
         if (is_lame_internal_flags_valid(gfc)) {
             flush_bitstream(gfc);
+            /* if user specifed buffer size = 0, dont check size */
+            if (mp3buffer_size == 0)
+                mp3buffer_size = INT_MAX;
             rc = copy_buffer(gfc, mp3buffer, mp3buffer_size, 1);
             save_gain_values(gfc);
         }
@@ -2138,7 +2163,13 @@ lame_encode_flush(lame_global_flags * gfp, unsigned char *mp3buffer, int mp3buff
 
         mp3buffer += imp3;
         mp3count += imp3;
-        frames_left -= ((frame_num != gfc->ov_enc.frame_number) ? 1 : 0);
+        {   /* even a single pcm sample can produce several frames!
+             * for example: 1 Hz input file resampled to 8 kHz mpeg2.5
+             */
+            int const new_frames = gfc->ov_enc.frame_number - frame_num;
+            if (new_frames > 0)
+                frames_left -=  new_frames;
+        }
     }
     /* Set esv->mf_samples_to_encode to 0, so we may detect
      * and break loops calling it more than once in a row.
@@ -2153,7 +2184,7 @@ lame_encode_flush(lame_global_flags * gfp, unsigned char *mp3buffer, int mp3buff
     mp3buffer_size_remaining = mp3buffer_size - mp3count;
     /* if user specifed buffer size = 0, dont check size */
     if (mp3buffer_size == 0)
-        mp3buffer_size_remaining = 0;
+        mp3buffer_size_remaining = INT_MAX;
 
     /* mp3 related stuff.  bit buffer might still contain some mp3 data */
     flush_bitstream(gfc);
@@ -2168,7 +2199,7 @@ lame_encode_flush(lame_global_flags * gfp, unsigned char *mp3buffer, int mp3buff
     mp3buffer_size_remaining = mp3buffer_size - mp3count;
     /* if user specifed buffer size = 0, dont check size */
     if (mp3buffer_size == 0)
-        mp3buffer_size_remaining = 0;
+        mp3buffer_size_remaining = INT_MAX;
 
     if (gfp->write_id3tag_automatic) {
         /* write a id3 tag to the bitstream */
@@ -2218,6 +2249,7 @@ lame_close(lame_global_flags * gfp)
             ret = -3;
         }
         if (NULL != gfc) {
+            gfc->lame_init_params_successful = 0;
             gfc->class_id = 0;
             /* this routine will free all malloc'd data in gfc, and then free gfc: */
             freegfc(gfc);
@@ -2295,6 +2327,53 @@ lame_mp3_tags_fid(lame_global_flags * gfp, FILE * fpStream)
 }
 
 
+static int
+lame_init_internal_flags(lame_internal_flags* gfc)
+{
+    if (NULL == gfc)
+        return -1;
+
+    gfc->cfg.vbr_min_bitrate_index = 1; /* not  0 ????? */
+    gfc->cfg.vbr_max_bitrate_index = 13; /* not 14 ????? */
+    gfc->cfg.decode_on_the_fly = 0;
+    gfc->cfg.findReplayGain = 0;
+    gfc->cfg.findPeakSample = 0;
+
+    gfc->sv_qnt.OldValue[0] = 180;
+    gfc->sv_qnt.OldValue[1] = 180;
+    gfc->sv_qnt.CurrentStep[0] = 4;
+    gfc->sv_qnt.CurrentStep[1] = 4;
+    gfc->sv_qnt.masking_lower = 1;
+
+    /* The reason for
+     *       int mf_samples_to_encode = ENCDELAY + POSTDELAY;
+     * ENCDELAY = internal encoder delay.  And then we have to add POSTDELAY=288
+     * because of the 50% MDCT overlap.  A 576 MDCT granule decodes to
+     * 1152 samples.  To synthesize the 576 samples centered under this granule
+     * we need the previous granule for the first 288 samples (no problem), and
+     * the next granule for the next 288 samples (not possible if this is last
+     * granule).  So we need to pad with 288 samples to make sure we can
+     * encode the 576 samples we are interested in.
+     */
+    gfc->sv_enc.mf_samples_to_encode = ENCDELAY + POSTDELAY;
+    gfc->sv_enc.mf_size = ENCDELAY - MDCTDELAY; /* we pad input with this many 0's */
+    gfc->ov_enc.encoder_padding = 0;
+    gfc->ov_enc.encoder_delay = ENCDELAY;
+
+    gfc->ov_rpg.RadioGain = 0;
+    gfc->ov_rpg.noclipGainChange = 0;
+    gfc->ov_rpg.noclipScale = -1.0;
+
+    gfc->ATH = lame_calloc(ATH_t, 1);
+    if (NULL == gfc->ATH)
+        return -2;      /* maybe error codes should be enumerated in lame.h ?? */
+
+    gfc->sv_rpg.rgdata = lame_calloc(replaygain_t, 1);
+    if (NULL == gfc->sv_rpg.rgdata) {
+        return -2;
+    }
+    return 0;
+}
 
 /* initialize mp3 encoder */
 #if DEPRECATED_OR_OBSOLETE_CODE_REMOVED
@@ -2304,20 +2383,12 @@ static
 int
 lame_init_old(lame_global_flags * gfp)
 {
-    lame_internal_flags *gfc;
-    SessionConfig_t *cfg;
-
     disable_FPE();      /* disable floating point exceptions */
 
     memset(gfp, 0, sizeof(lame_global_flags));
 
     gfp->class_id = LAME_ID;
 
-    if (NULL == (gfc = gfp->internal_flags = calloc(1, sizeof(lame_internal_flags))))
-        return -1;
-
-    cfg = &gfc->cfg;
-
     /* Global flags.  set defaults here for non-zero values */
     /* see lame.h for description */
     /* set integer values to -1 to mean that LAME will compute the
@@ -2344,25 +2415,16 @@ lame_init_old(lame_global_flags * gfp)
 
     gfp->VBR = vbr_off;
     gfp->VBR_q = 4;
-    gfp->ATHcurve = -1;
     gfp->VBR_mean_bitrate_kbps = 128;
     gfp->VBR_min_bitrate_kbps = 0;
     gfp->VBR_max_bitrate_kbps = 0;
     gfp->VBR_hard_min = 0;
-    cfg->vbr_min_bitrate_index = 1; /* not  0 ????? */
-    cfg->vbr_max_bitrate_index = 13; /* not 14 ????? */
 
     gfp->quant_comp = -1;
     gfp->quant_comp_short = -1;
 
     gfp->msfix = -1;
 
-    gfc->sv_qnt.OldValue[0] = 180;
-    gfc->sv_qnt.OldValue[1] = 180;
-    gfc->sv_qnt.CurrentStep[0] = 4;
-    gfc->sv_qnt.CurrentStep[1] = 4;
-    gfc->sv_qnt.masking_lower = 1;
-
     gfp->attackthre = -1;
     gfp->attackthre_s = -1;
 
@@ -2370,38 +2432,17 @@ lame_init_old(lame_global_flags * gfp)
     gfp->scale_left = 1;
     gfp->scale_right = 1;
 
-    gfp->athaa_type = -1;
+    gfp->ATHcurve = -1;
     gfp->ATHtype = -1;  /* default = -1 = set in lame_init_params */
     /* 2 = equal loudness curve */
     gfp->athaa_sensitivity = 0.0; /* no offset */
+    gfp->athaa_type = -1;
     gfp->useTemporal = -1;
     gfp->interChRatio = -1;
 
-    /* The reason for
-     *       int mf_samples_to_encode = ENCDELAY + POSTDELAY;
-     * ENCDELAY = internal encoder delay.  And then we have to add POSTDELAY=288
-     * because of the 50% MDCT overlap.  A 576 MDCT granule decodes to
-     * 1152 samples.  To synthesize the 576 samples centered under this granule
-     * we need the previous granule for the first 288 samples (no problem), and
-     * the next granule for the next 288 samples (not possible if this is last
-     * granule).  So we need to pad with 288 samples to make sure we can
-     * encode the 576 samples we are interested in.
-     */
-    gfc->sv_enc.mf_samples_to_encode = ENCDELAY + POSTDELAY;
-    gfc->ov_enc.encoder_padding = 0;
-    gfc->sv_enc.mf_size = ENCDELAY - MDCTDELAY; /* we pad input with this many 0's */
-
     gfp->findReplayGain = 0;
     gfp->decode_on_the_fly = 0;
 
-    gfc->cfg.decode_on_the_fly = 0;
-    gfc->cfg.findReplayGain = 0;
-    gfc->cfg.findPeakSample = 0;
-
-    gfc->ov_rpg.RadioGain = 0;
-    gfc->ov_rpg.noclipGainChange = 0;
-    gfc->ov_rpg.noclipScale = -1.0;
-
     gfp->asm_optimizations.mmx = 1;
     gfp->asm_optimizations.amd3dnow = 1;
     gfp->asm_optimizations.sse = 1;
@@ -2413,6 +2454,14 @@ lame_init_old(lame_global_flags * gfp)
     gfp->report.debugf = &lame_report_def;
     gfp->report.errorf = &lame_report_def;
     gfp->report.msgf = &lame_report_def;
+
+    gfp->internal_flags = lame_calloc(lame_internal_flags, 1);
+
+    if (lame_init_internal_flags(gfp->internal_flags) < 0) {
+        freegfc(gfp->internal_flags);
+        gfp->internal_flags = 0;
+        return -1;
+    }
     return 0;
 }
 
@@ -2425,7 +2474,7 @@ lame_init(void)
 
     init_log_table();
 
-    gfp = calloc(1, sizeof(lame_global_flags));
+    gfp = lame_calloc(lame_global_flags, 1);
     if (gfp == NULL)
         return NULL;
 
