@@ -1,5 +1,7 @@
 /*
- * Copyright (C) 2000 Albert L. Faber
+ * Copyright (C) 1999-2010 The L.A.M.E. project
+ *
+ * Initially written by Michael Hipp, see also AUTHORS and README.
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,11 +33,11 @@ extern real muls[27][64];
 
 
 int     head_check(unsigned long head, int check_layer);
-int     decode_header(struct frame *fr, unsigned long newhead);
-void    print_header(struct frame *fr);
-void    print_header_compact(struct frame *fr);
+int     decode_header(PMPSTR mp, struct frame *fr, unsigned long newhead);
 unsigned int getbits(PMPSTR mp, int number_of_bits);
 unsigned int getbits_fast(PMPSTR mp, int number_of_bits);
+unsigned char get_leq_8_bits(PMPSTR mp, unsigned int number_of_bits);
+unsigned short get_leq_16_bits(PMPSTR mp, unsigned int number_of_bits);
 int     set_pointer(PMPSTR mp, long backstep);
 
 #endif
