@@ -1,5 +1,7 @@
 /*
- * Copyright (C) 2000 Albert L. Faber
+ * Copyright (C) 1999-2010 The L.A.M.E. project
+ *
+ * Initially written by Michael Hipp, see also AUTHORS and README.
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -29,8 +31,9 @@ struct al_table2 {
 
 
 
-void    init_layer2(void);
-int     do_layer2(PMPSTR mp, unsigned char *pcm_sample, int *pcm_point);
+void    hip_init_tables_layer2(void);
+int     decode_layer2_sideinfo(PMPSTR mp);
+int     decode_layer2_frame(PMPSTR mp, unsigned char *pcm_sample, int *pcm_point);
 
 
 #endif
